#include <WProgram.h>  //needed so standard Arduino defines are available in defines.h
#include <EEPROM.h>
#include <LiquidCrystal.h>
#include <avr/pgmspace.h>
#include "defines.h"

/* User defined constants */
#define BUTTON_UP_PIN     12
#define BUTTON_LEFT_PIN   13
#define BUTTON_DOWN_PIN   9
#define BUTTON_RIGHT_PIN  10
#define BUZZER_PIN        11
#define LCD_RS A5
#define LCD_RW A4
#define LCD_E  A3
#define LCD_D0 A2
#define LCD_D1 A1
#define LCD_D2 2
#define LCD_D3 A0
#define LCD_D4 5
#define LCD_D5 3
#define LCD_D6 7
#define LCD_D7 8
#define PRIMARY_POWER 4
#define LCD_BACKLIGHT 6

//initial time/date values
//if these are changed also change MAGIC_NUMBER so they will be used
#define START_DAY    TUE  //0(Sunday)-6(Saturday)
#define START_HOUR   6 //0-23
#define START_MINUTE 32  //0-59
#define START_SECONDS 50

#define ALARM_ONETIME_HOUR    10
#define ALARM_ONETIME_MINUTE  0
#define ALARM_ONETIME_ENABLED false

#define ALARM_PERSISTENT_HOUR    6
#define ALARM_PERSISTENT_MINUTE  15
#define ALARM_PERSISTENT_ENABLED false

#define MAGIC_NUMBER 58  //number that a byte of EEPROM is compared to in order to determine is one-time initializations should be done
#define VERSION 1  //current version
#define BUZZER_PRESCALER 3  //lower number higher pitch;0-mute,1-1,2-8,3-32,4-64,5-128,6-256,7-1024
#define BUZZER_COMPARE 61    //lower number higher pitch
#define MUTE_TIME 30000  //time in ms the buzzer is silenced every time a button is pressed
#define SNOOZE_TIME_DEF 7  //default time in min of the snooze delay
#define SNOOZE_TIME_SEC_DEF 30  //default time in seconds to delay in addition to SNOOZE_TIME, should be less than 60
#define SNOOZE_NUMBER_DEF 2  //default number of times you're allowed to hit the snooze
#define LCD_BACKLIGHT_DEFAULT 16  //default backlight brighness, 128 is the brightest
#define AWAKE_LINES 4  //number of lines that need be cleared to declare the user awake

/* Constants that should not be changed*/
#define ARDUINO_OFFSET 43  //on error, the filename and line number are displayed, the arduino environment adds a few lines to the beginning of every sketch which throws off the line number, so to display the correct line number the number of lines the arduino enviroment adds has to be subtracted off.  I call this number, for lack of a better name, ARDUINO_OFFSET
#define TETRIS_DOUBLE_BUTTON_TIME 200  //time in ms that two buttons pressed will be declared pressed together

#define HALF_32BIT 2147483648  //2^32/2, one half of the full amout of the 32 bit millis() time
#define MAX_BOARD_HEIGHT 80

#define error print_error(250,__LINE__)

#define BUTTON_SCALER 0x03  //scaler for button timer, 16MHz clock 16 bit timer, two bits means 4.1ms timer period multiplies by 4 becoming 16ms
#define BUTTON_MASK 0x07  //0x03 scaler, 3 presses in a row = 3 * 16ms = 49ms (close to 50ms)

/* Global variables */
//boolean (1-byte) 6 6
boolean wrap_time = 0;  //indicates we're subtracting HALF_32BIT from the millis() time
boolean update_time = false;  //set to true every second to indicite it's time to update the displays
boolean recently_reset = true;  //indicates the processor has been reset, cleared by pressing any button, the randomness of when that button is pressed seeds the random number generator
boolean trigger_alarm = false;  //triggers checking if the alarm should go off, happens every minute
boolean paused = false;  //indicates display is paused so something just displayed will linger without stopping the processor entirely
boolean ignore_button = false;
boolean powered = true;  //indicates primary power is present, used to tell if power has just been removed so the backlight can be left on for a while
boolean new_piece_needed = false;  //indicates a new piece is needed, i.e. the last piece has locked down or the beginning of a new game
boolean wait_for_new_game = true;  //wait for a button press to start next game
boolean illuminated = true;  //indicates backlight is currently on
//boolean thicker = true;  //indicates thicker, 4x4 dot blocks are used to make the pieces, a width of only 8 columns must be used
//boolean two_six = true;  //indicates we are using 2 characters for displaying the piece, and 6 characters for displaying the rubble
boolean save_eeprom = false;  //indicates EEPROM has changed and needs to be saved at midnight

//1-byte 7 13
byte button_timer = 0;  //incremented every timer interrupt, used to divide timer with BUTTON_SCALER
byte volatile button_pressed_count = 0; //count of button presses, used to determine if action needs to be taken, 0 when idle; accessed by interrupt and main routine so needs to be volatile
byte current_button = 0; //bit mapped record of a button being pressed, used with last_button to detect edge of a button press so a button only registers once before it it released
byte last_button = 0;    //but mapped record of the last button pressed, used with curent_button to detect edge of a button press so a button only registers once before it it released
byte operand1 = 0;  //numbers to be multiplied together to prove you're awake
byte operand2 = 0;  //numbers to be multiplied together to prove you're awake
byte snooze_count = SNOOZE_NUMBER_DEF;  //count of number of snooze hits left
byte piece_id = 0;  //which piece is currently in play, number from 0-6
byte piece_rot = 0;  //piece rotation, number from 0-3
byte piece_x = 0;  //piece location x (horizontal) coordinate
byte piece_y = 0;  //piece location y (vertical) coordinate
byte stack_height = 0;  //current height of the fallen pieces
byte lines_cleared = 0;  //the number of lines that have been cleared
byte snoooze_remainder = 0;  //how much extra seconds of snooze time that needs to be kept track of
byte previous_button = 0;  //used in check_double to guard against double button presses as opposed to pressing two buttons at once

//2-byte 4 8 21
enum set_time_enum set_time_place = SET_MODE;  //indicatees which value we're changing while in time set mode
enum time_mode_enum time_mode = RUN_MODE;  //indicates what time mode we're in, run, time set, one-time alarm set, or persistent (M-F) alarm set
unsigned result = 13;  //result of the numbers multiplied together
unsigned guess = 0;  //current guess of the numbers multiplied together
enum turnoff_enum turnoff = TETRIS;  //weather the device to prove you're awake is math or playing tetris
unsigned drop_interval = 500;  //interval in ms between when a piece if dropped one row
unsigned board_mask = 0;  //mask that makes the edges of the board

//4-byte 4 16 37
unsigned long current_time = 0;  //the current time
unsigned long previous_time = 0;  //the previous time used to know how much time has passed in time struct updated
unsigned long mute = 0;  //MUTE_TIME miliseconds in to the future, used to temporarily mute the alarm every time a button is pressed
unsigned long pause_end_time = 0;  //when pausing should stop, in milliseconds
unsigned long previous_drop_time = 0;  //record of last time a piece was dropped to compare current time to in order to determine if it's time to drop down a row
unsigned long last_tetris_button = 0;  //time when a button was pressed used to decide if two buttons were pressed together

//multi-byte 94 131
eeprom_type eeprom;  //everything stored in persistent EEPROM memory 3
time_type time;  //where all the current time and date info is kept 8
alarm_type alarm_onetime;  //alarm that happens once and turns off 3
alarm_type alarm_persistent;  //alarm that happens Monday through Friday 3
byte button_count[4];  //used to count how many ticks in a row a button was pressed 4
byte volatile buttons[4];  //count of actual button presses, increments once per press, all 0 when idle; accessed by interrupt and main routine so needs to be volatile 4
char outstring[30]; //temporay buffer 30
char outstring2[30]; //temporary buffer 30
byte stops[SET_HUNDREDTHS+1];//where the blinking cursor indicator should stop while setting time and alarms 9
unsigned board[MAX_BOARD_HEIGHT];  //the board on which we're playing, includes fallen or set blocks and the current piece in play 160
unsigned board_thick[MAX_BOARD_HEIGHT];  //the board with everything made thicker, used to simplify the display
byte custom[8];  //used to create custom characters for the LCD display
unsigned pieces[7][4]={  //pieces[piece_id][piece_rot] 56
  {PIECE_I_0,PIECE_I_1,PIECE_I_2,PIECE_I_3},
  {PIECE_T_0,PIECE_T_1,PIECE_T_2,PIECE_T_3},
  {PIECE_J_0,PIECE_J_1,PIECE_J_2,PIECE_J_3},
  {PIECE_L_0,PIECE_L_1,PIECE_L_2,PIECE_L_3},
  {PIECE_Z_0,PIECE_Z_1,PIECE_Z_2,PIECE_Z_3},
  {PIECE_S_0,PIECE_S_1,PIECE_S_2,PIECE_S_3},
  {PIECE_O_0,PIECE_O_1,PIECE_O_2,PIECE_O_3}};
unsigned adj_piece[4];  //place to put an adjusted piece in preparation for comparing to the board

//extern unsigned long timer0_millis;

/*
LiquidCrystal(rs, enable, d4, d5, d6, d7)
LiquidCrystal(rs, rw, enable, d4, d5, d6, d7)
LiquidCrystal(rs, enable, d0, d1, d2, d3, d4, d5, d6, d7)
LiquidCrystal(rs, rw, enable, d0, d1, d2, d3, d4, d5, d6, d7)
*/
LiquidCrystal lcd(LCD_RS,LCD_RW,LCD_E,LCD_D0,LCD_D1,LCD_D2,LCD_D3,LCD_D4,LCD_D5,LCD_D6,LCD_D7);

void setup() {

  //Serial.begin(9600);
  lcd.begin(16, 2);
  
  analogWrite(LCD_BACKLIGHT, 16);

  strcpy_P(outstring2, digit_s);
  sprintf(outstring,outstring2,availableMemory());
  print_top(outstring);
  strcpy_P(outstring2, version_s);
  sprintf(outstring,outstring2,VERSION);
  print_bot(outstring);

  get_eeprom();

  if(eeprom.magic_number != MAGIC_NUMBER){
    factory_init();
  }

  analogWrite(LCD_BACKLIGHT, eeprom.lcd_backlight);
  
  set_time_characters();

  //make the button pins inputs
  pinMode(BUTTON_UP_PIN, INPUT);           // set pin to input
  pinMode(BUTTON_DOWN_PIN, INPUT);           // set pin to input
  pinMode(BUTTON_LEFT_PIN, INPUT);           // set pin to input
  pinMode(BUTTON_RIGHT_PIN, INPUT);           // set pin to input
  pinMode(PRIMARY_POWER, INPUT);
  //enable the internal pullup resistors
  digitalWrite(BUTTON_UP_PIN, HIGH);       // turn on pullup resistors
  digitalWrite(BUTTON_DOWN_PIN, HIGH);       // turn on pullup resistors
  digitalWrite(BUTTON_LEFT_PIN, HIGH);       // turn on pullup resistors
  digitalWrite(BUTTON_RIGHT_PIN, HIGH);       // turn on pullup resistors
  digitalWrite(PRIMARY_POWER, HIGH);       // turn on pullup resistors

  zero_outs(board,sizeof(board));
  init_board();
  zero_outs(button_count,sizeof(button_count));
  zero_outvs(buttons,sizeof(buttons));

  previous_drop_time = adjusted_millis();

  //initialize timer interrupt
  TCCR1B = bit(CS10);  //no prescaling
  TIMSK1 = bit(TOIE1);  //overflow interrupt enable

  //alarm
  TCCR2A = bit(COM2A0) | bit(WGM21);  //toggle OC2A, Clear Timer on Compare Match
  TCCR2B = BUZZER_PRESCALER;
  OCR2A = BUZZER_COMPARE;

  pinMode(BUZZER_PIN,INPUT);//turn alarm off initially

  sei();  //start interrupts

}//void setup()

void loop() {
  current_time = adjusted_millis();
  time.ms += current_time - previous_time;

  //take care of the actual time and all the incrementation involved
  do{
  if(time.ms >= 1000){  //there's a (ridiculously small) chance several seconds have gone by
    time.ms -= 1000;    //so only subtract one second of time just in case
    time.second++;
    update_time = true;
    if(time.second >= 60){
      time.second = 0;
      time.minute++;
      trigger_alarm = true;
      if(time.minute >= 60){
        time.minute = 0;
        time.hour++;
        if(time.hour >= 24){  //new day
          time.hour = 0;
          time.day = (day_enum)(time.day + 1);//time.day++;
          if(time.day > SAT)
            time.day = SUN;
          if(save_eeprom)
            set_eeprom();
        }
      }
    }
  }
  }while(time.ms > 1000);

  previous_time = current_time;

  if (update_time){
    if((time_mode == ALARM_TURNOFF || time_mode == ALARM_SNOOZE_TURNOFF) && turnoff == TETRIS){
    }else{  //if playing tetris do nothing, otherwise display the time
      display_time();
    }
    update_time = false;

    if(!digitalRead(PRIMARY_POWER)){
      powered = true;
    }else{
      if(powered){  //if powered is true primary power has just been removed, don't turn off backlight until MUTE_TIME has elapsed
        mute = adjusted_millis() + MUTE_TIME;
      }
      powered = false;
    }

    if(!digitalRead(PRIMARY_POWER) || (adjusted_millis() < mute)){  //primary power is present, turn on backlight
      analogWrite(LCD_BACKLIGHT, eeprom.lcd_backlight);
      illuminated = true;
    }else{
      analogWrite(LCD_BACKLIGHT, 0);
      illuminated = false;
    }
  }

  //check the alarms
  if(trigger_alarm && alarm_onetime.hour == time.hour && alarm_onetime.minute == time.minute && alarm_onetime.enabled){
    alarm_onetime.enabled = false;
    check_sleep();
  }
  if(trigger_alarm && alarm_persistent.hour == time.hour && alarm_persistent.minute == time.minute && time.day >= 1 && time.day <= 5 && alarm_persistent.enabled){
    check_sleep();
  }

  //check if the mute time has passed (user has fallen back to sleep)
  if(adjusted_millis() > mute){
    if(time_mode == ALARM_TURNOFF){
      time_mode = ALARM_TURNOFF_BUZZING;
    }
    if(time_mode == ALARM_SNOOZE_TURNOFF){
      time_mode = ALARM_SNOOZED;
      set_time_characters();
    }
  }

  //make the alarm sound
  if(time_mode == ALARM_SNOOZE_BUZZING){
    if(time.ms < 500)
      pinMode(BUZZER_PIN,OUTPUT);//turn alarm on
    else
      pinMode(BUZZER_PIN,INPUT);//turn alarm off
  } else if(time_mode == ALARM_TURNOFF_BUZZING){
    if(time.ms < 250 || (time.ms > 500 && time.ms < 750))
      pinMode(BUZZER_PIN,OUTPUT);//turn alarm on
    else
      pinMode(BUZZER_PIN,INPUT);//turn alarm off
  } else{
    pinMode(BUZZER_PIN,INPUT);//turn alarm off
  }

  //check to see if the alarm should be turned off
  if((time_mode == ALARM_SNOOZE_TURNOFF || time_mode == ALARM_TURNOFF) && (guess == result || lines_cleared >= eeprom.tetris_clear_lines)){
    time_mode = RUN_MODE;
    set_time_place = SET_MODE;
    guess = 0;
    snooze_count = eeprom.snooze_number;
    alarm_onetime.enabled = false;
    wait_for_new_game = true;
    set_time_characters();
    snoooze_remainder = 0;
  }

  //check if a new tetris piece is needed
  if(new_piece_needed){
    new_piece_needed = false;

    //count the height of the fallen pieces before adding another piece
    stack_height = 0;
    while(board[stack_height] & ~board_mask)
      stack_height++;

    //remove any full lines
    for(byte i = 1; i < stack_height; i++){
      if((board[i] | board_mask) == 0xFFFF){  //check for a full line
        for(byte j = i; j < stack_height+1; j++){  //make the stack above the full line drop down one line
          board[j] = board[j+1];
        }
        i--;  //and since we've removed a line decrement i
        lines_cleared++;
      }
    }

    get_new_piece();

    if(!piece_fit()){
      wait_for_new_game = true;
    }

    set_piece();
  }  //if(new_piece_needed)

  //adjust the time if need be
  if(previous_time >= HALF_32BIT && !paused) {  //we've passed the half way mark in time so adjust the current_time and (if not paused)
    previous_drop_time -= HALF_32BIT;     //tell the adjusted_millis() function to either start or stop adjusting
    previous_time -= HALF_32BIT;     //tell the adjusted_millis() function to either start or stop adjusting
    wrap_time ^= 1;
  }

  //drop the piece
  if((adjusted_millis() >= previous_drop_time + drop_interval) && !wait_for_new_game &&
     (time_mode == ALARM_TURNOFF || time_mode == ALARM_SNOOZE_TURNOFF) && turnoff == TETRIS){

    previous_drop_time = adjusted_millis() + drop_interval;

    remove_piece();             //remove the piece
    piece_y--;                  //drop the piece
    if(!piece_fit()){           //test to see if it fits and if not
      piece_y++;                //put the piece back and
      new_piece_needed = true;  //indicate that a new piece is needed
    }
    set_piece();                //put the piece back
  }  //if((adjusted_millis() >= previous_drop_time + drop_interval) && !wait_for_new_game)

  if(button_pressed_count){  //a button is pressed, take care of it
    //ignore_button = false;

    if(recently_reset){  //the time of the first button press randomizes the random number generator
      recently_reset = false;
      randomSeed(millis());
      ignore_button = true;
    }

    mute = adjusted_millis() + MUTE_TIME;

    //adjust time_mode if necessary and ignore the button press
    if(!illuminated){  //if a button is pressed when not illuminated ignore it
      ignore_button = true;
    }else if(time_mode == ALARM_SNOOZE_BUZZING){
      ignore_button = true;
      time_mode = ALARM_SNOOZED;
      if(eeprom.snooze_relative)
        add_snooze_time();
    }else if(time_mode == ALARM_SNOOZED){
      ignore_button = true;
      time_mode = ALARM_SNOOZE_TURNOFF;
      adjust_for_display();
    }else if(time_mode == ALARM_TURNOFF_BUZZING){
      ignore_button = true;
      time_mode = ALARM_TURNOFF;
      adjust_for_display();
    }

    //wait for a button press to start a new game, this helps seed the random number generator with a random value
    if(wait_for_new_game){
      wait_for_new_game = false;

      randomSeed(millis());
      random();//first random number after randomSeed is always 0

      zero_outs(board,sizeof(board));
      init_board();

      new_piece_needed = true;

      lines_cleared = 0;

      ignore_button = true;
    }  //if(wait_for_new_game)

    if(buttons[BUTTON_SELECT]){  //aka BUTTON_DOWN (drop)
      buttons[BUTTON_SELECT]--;
      button_pressed_count--;

      if(!ignore_button){
        switch (time_mode){
          case RUN_MODE:
            time_mode = TIME_SET_MODE;
            set_time_place = SET_MODE;
            adjust_for_display();
            break;
          case PERSISTENT_ALARM_SET_MODE:
          case ONETIME_ALARM_SET_MODE:
          case TIME_SET_MODE:
            if(set_time_place < SET_HUNDREDTHS){
              set_time_place = (set_time_enum)((byte)set_time_place + 1);  //c++ doesn't allow you to increment enums, this line should be simply set_time_place++;
            }else{
              time_mode = RUN_MODE;
              save_eeprom = true;
            }
            break;
          case ALARM_TURNOFF:
          case ALARM_SNOOZE_TURNOFF:
            if(turnoff == MATH){
              if(set_time_place < SET_MINUTE10)
                set_time_place = (set_time_enum)((byte)set_time_place + 1);
            }else{
              remove_piece();           //remove the piece
              while(piece_fit())        //drop the piece as far as it will go
                piece_y--;
              piece_y++;                //once we've found where it won't fit bump it up one row
              set_piece();              //put the piece back
              new_piece_needed = true;  //indicate that a new piece is needed
            }
            break;
          default:
            error;
        }//switch (time_mode)
      }//if(!ignore_button)
    }  //if(buttons[BUTTON_SELECT])  //select button pressed

    if(buttons[BUTTON_GOBACK]){  //aka BUTTON_UP (rotate)
      buttons[BUTTON_GOBACK]--;
      button_pressed_count--;

      if(!ignore_button){
        switch (time_mode){
          case RUN_MODE:
            main_menu();
            lcd.noBlink();
            //set_eeprom();  //if goback button pressed while not in time set mode, save EEPROM
            //save_eeprom = true;
            //timer0_millis += (HALF_32BIT-20000);  //used to test the adjusted_millis and the wrap function work
            break;
          case PERSISTENT_ALARM_SET_MODE:
          case ONETIME_ALARM_SET_MODE:
          case TIME_SET_MODE:
            if(set_time_place > SET_MODE)
              set_time_place = (set_time_enum)((byte)set_time_place - 1);  //c++ doesn't allow you to increment enums, this line should be simply set_time_place--;
            else{
              time_mode = RUN_MODE;
              //set_eeprom();
              save_eeprom = true;
            }
            break;
          case ALARM_TURNOFF:
          case ALARM_SNOOZE_TURNOFF:
            if(turnoff == MATH){
              if(set_time_place > SET_MODE)
                set_time_place = (set_time_enum)((byte)set_time_place - 1);
            }else{
              remove_piece();                   //remove the piece
              piece_rot = (piece_rot+1) % 4;    //rotate the piece
              if(!piece_fit())                  //test to see if it fits and if not
                piece_rot = (piece_rot-1) % 4;  //rotate back,
              set_piece();                      //put the piece back
            }
            break;
          default:
            error;
        }//switch (time_mode)
      }//if(!ignore_button)
    }  //if(buttons[BUTTON_GOBACK])  //go back button pressed

    if(buttons[BUTTON_MENUUP]){  //aka BUTTON_RIGHT (move piece right while screen vertical)
      buttons[BUTTON_MENUUP]--;
      button_pressed_count--;

      if(!ignore_button){
        switch (time_mode){
          case RUN_MODE:
//            eeprom.lcd_backlight += 32;
            if(eeprom.lcd_backlight)
              eeprom.lcd_backlight <<= 1;
            else
              eeprom.lcd_backlight = 1;
            analogWrite(LCD_BACKLIGHT, eeprom.lcd_backlight);
            save_eeprom = true;
            break;
          case TIME_SET_MODE:
          case ONETIME_ALARM_SET_MODE:
          case PERSISTENT_ALARM_SET_MODE:
            adjust_time(1);
            break;
          case ALARM_TURNOFF:
          case ALARM_SNOOZE_TURNOFF:
            if(turnoff == MATH){
              adjust_time(1);
            }else{
              remove_piece();   //remove the piece
              piece_x++;        //move the piece to the left
              if(!piece_fit())  //test to see if it fits and if not
                piece_x--;      //move back
              set_piece();      //put the piece back

              check_double(BUTTON_MENUUP);
            }
            break;
          default:
            error;
       }//switch (time_mode)
      }//if(!ignore_button)
    }

    if(buttons[BUTTON_MENUDOWN]){  //aka BUTTON_LEFT (move piece left while screen vertical)
      buttons[BUTTON_MENUDOWN]--;
      button_pressed_count--;

      if(!ignore_button){
        switch (time_mode){
          case RUN_MODE:
//            eeprom.lcd_backlight -= 32;
            if(eeprom.lcd_backlight)
              eeprom.lcd_backlight >>= 1;
            else
              eeprom.lcd_backlight = 0x80;
            analogWrite(LCD_BACKLIGHT, eeprom.lcd_backlight);
            save_eeprom = true;
            break;
          case TIME_SET_MODE:
          case ONETIME_ALARM_SET_MODE:
          case PERSISTENT_ALARM_SET_MODE:
            adjust_time(-1);
            break;
          case ALARM_TURNOFF:
          case ALARM_SNOOZE_TURNOFF:
            if(turnoff == MATH){
              adjust_time(-1);
            }else{
              remove_piece();   //remove the piece
              piece_x--;        //move the piece to the left
              if(!piece_fit())  //test to see if it fits and if not
                piece_x++;      //move back
              set_piece();      //put the piece back

              check_double(BUTTON_MENUDOWN);
            }
            break;
          default:
            error;
        }//switch (time_mode)
      }//if(!ignore_button)
    }

    ignore_button = false;

  }//if(button_pressed_count)  //a button is pressed, take care of it

  if((time_mode == ALARM_TURNOFF || time_mode == ALARM_SNOOZE_TURNOFF) && turnoff == TETRIS){
    display_tetris();  //display the tetris screen
  }

}//void loop()

void main_menu(void){
  enum mainmenu_enum menu = MAINMENU_THICKER;
  enum button_enum button;
   //byte submenu;

   lcd.blink();

   while(1){
   /*Choose a menu*/
   do{   //cycle through the display menu selections until one is selected
      switch(menu){  //the order of this switch statement doesn't set the order of the menu, the order of the enum does that
         case MAINMENU_THICKER:
            strcpy_P(outstring, mthicker_s);
            print_top(outstring);
            strcpy_P(outstring, (char*)pgm_read_word(&(thicker_s[eeprom.thicker])));
            print_bot(outstring);
            break;
         case MAINMENU_TWO_SIX:
            strcpy_P(outstring, mtwosix_s);
            print_top(outstring);
            strcpy_P(outstring, (char*)pgm_read_word(&(twosix_s[eeprom.two_six])));
            print_bot(outstring);
            break;
         case MAINMENU_SNOOZE_TIME:
            strcpy_P(outstring, snoozetime_s);
            print_top(outstring);
            print_bot(eeprom.snooze_time_min) ;
            break;
         case MAINMENU_SNOOZE_TIME_SEC:
            strcpy_P(outstring, snoozetimesec_s);
            print_top(outstring);
            print_bot(eeprom.snooze_time_sec) ;
            break;
         case MAINMENU_SNOOZE_NUMBER:
            strcpy_P(outstring, snoozenumber_s);
            print_top(outstring);
            print_bot(eeprom.snooze_number) ;
            break;
         case MAINMENU_TETRIS_CLEAR_LINES:
            strcpy_P(outstring, clearlines_s);
            print_top(outstring);
            print_bot(eeprom.tetris_clear_lines) ;
            break;
         case MAINMENU_VERSION:
            strcpy_P(outstring, version_s);
            print_top(outstring);
            print_bot(VERSION) ;
            break;
         case MAINMENU_SNOOZE_RELATIVE:
            strcpy_P(outstring, snoozerelative_s);
            print_top(outstring);
            strcpy_P(outstring, (char*)pgm_read_word(&(relative_s[eeprom.snooze_relative])));
            print_bot(outstring);
            break;
         case MAINMENU_FACT_INIT:
            strcpy_P(outstring, factinit_s);
            print_top(outstring);
            print_bot();
            break;
         case MAINMENU_SAVE_EEPROM:
            strcpy_P(outstring, saveeeprom_s);
            print_top(outstring);
            print_bot();
            break;
         case MAINMENU_GET_EEPROM:
            strcpy_P(outstring, mgeteeprom_s);
            print_top(outstring);
            print_bot();
            break;
         case MAINMENU_EXTRAS_ONOFF:
            strcpy_P(outstring, extras_s);
            print_top(outstring);
            strcpy_P(outstring, (char*)pgm_read_word(&(onoff_s[eeprom.extras])));
            print_bot(outstring);
            break;
         default:
            error;
            break;
      }

      lcd.setCursor(1,0);  //(column, row)

      button = getbutton();

      switch(button){
        case BUTTON_MENUUP:
          menu = (mainmenu_enum)(menu - 1);
          break;
        case BUTTON_MENUDOWN:
          menu = (mainmenu_enum)(menu + 1);
          break;
        case BUTTON_GOBACK:
          return;
        case BUTTON_SELECT:
          break;  //do nothing
        default:
          error;
      }

      menu = (mainmenu_enum)mod(menu, 0, MAINMENU_LAST-1);

   }while(button != BUTTON_SELECT);

   //act on menu selection
   switch(menu){
      case MAINMENU_THICKER:
         strcpy_P(outstring, mthicker_s);
         print_top(outstring);
         do{
            strcpy_P(outstring, (char*)pgm_read_word(&(thicker_s[eeprom.thicker])));
            print_bot_c(outstring);

            button = getbutton();

            if(button == BUTTON_MENUUP || button == BUTTON_MENUDOWN)
               eeprom.thicker ^= 1;

         }while(button != BUTTON_GOBACK);

         break;

      case MAINMENU_TWO_SIX:
         strcpy_P(outstring, mtwosix_s);
         print_top(outstring);
         do{
            strcpy_P(outstring, (char*)pgm_read_word(&(twosix_s[eeprom.two_six])));
            print_bot_c(outstring);

            button = getbutton();

            if(button == BUTTON_MENUUP || button == BUTTON_MENUDOWN)
               eeprom.two_six ^= 1;

         }while(button != BUTTON_GOBACK);

         break;

      case MAINMENU_SNOOZE_TIME:
         strcpy_P(outstring, snoozetime_s);
         print_top(outstring);

         eeprom.snooze_time_min = input_byte(eeprom.snooze_time_min);

         break;

      case MAINMENU_SNOOZE_TIME_SEC:
         strcpy_P(outstring, snoozetimesec_s);
         print_top(outstring);

         eeprom.snooze_time_sec = input_byte(eeprom.snooze_time_sec);

         break;

      case MAINMENU_SNOOZE_NUMBER:
         strcpy_P(outstring, snoozenumber_s);
         print_top(outstring);

         eeprom.snooze_number = input_byte(eeprom.snooze_number);

         break;

      case MAINMENU_TETRIS_CLEAR_LINES:
         strcpy_P(outstring, clearlines_s);
         print_top(outstring);

         eeprom.tetris_clear_lines = input_byte(eeprom.tetris_clear_lines);

         break;

      case MAINMENU_SNOOZE_RELATIVE:
         strcpy_P(outstring, snoozerelative_s);
         print_top(outstring);
         do{
            strcpy_P(outstring, (char*)pgm_read_word(&(relative_s[eeprom.snooze_relative])));
            print_bot_c(outstring);

            button = getbutton();

            if(button == BUTTON_MENUUP || button == BUTTON_MENUDOWN)
               eeprom.snooze_relative ^= 1;

         }while(button != BUTTON_GOBACK);

         break;

      case MAINMENU_FACT_INIT:
         eeprom.magic_number = 0;
         set_eeprom();
         strcpy_P(outstring, factinitt_s);
         print_top(outstring);
         strcpy_P(outstring, factinitb_s);
         print_bot(outstring);
         error;
         while(1);

         break;

      case MAINMENU_SAVE_EEPROM:
         set_eeprom();
         break;

      case MAINMENU_GET_EEPROM:
         get_eeprom();
         break;

      case MAINMENU_EXTRAS_ONOFF:
         strcpy_P(outstring, extras_s);
         print_top(outstring);
         do{
            strcpy_P(outstring, (char*)pgm_read_word(&(onoff_s[eeprom.extras])));
            print_bot_c(outstring);

            button = getbutton();

            if(button == BUTTON_MENUUP || button == BUTTON_MENUDOWN)
               eeprom.extras ^= 1;

         }while(button != BUTTON_GOBACK);

         break;
         
      case MAINMENU_VERSION:
        break;

      default:
         error;
         break;
   }  //switch(menu){}  //void output_cont_pulse_menu(void)
   }//while(1)
}  //void main_menu(void)

void display_time(){
  byte hour,hour_tens,minute,minute_tens;
  char day[3];
  char onoff[4];
  byte adjust;

  if(paused && (adjusted_millis() > pause_end_time))
    paused = false;

  if(!paused){
    lcd.noBlink();
    strcpy_P(day, (char*)pgm_read_word(&(days_week[time.day])));
    adjust = stops[set_time_place];
    set_customs();
    switch (time_mode){
      case RUN_MODE:
      case ALARM_SNOOZED:
      case ALARM_TURNOFF_BUZZING:
      case ALARM_SNOOZE_BUZZING:

        //prepare for the large numbers
        hour=time.hour;
        if(time.hour > 12)
          hour -= 12;
        if(!hour)
          hour += 12;
        hour_tens = hour / 10;
        hour -= (hour_tens*10);
        minute = time.minute;
        minute_tens = minute / 10;
        minute -= (minute_tens*10);

        //write the large numbers
        for(byte i=0; i<2; i++){
          lcd.setCursor(0,i);  //(column, row)
          if(hour_tens)
            write_number(hour_tens,i);  //write first digit of hour
          else
            write_number(10,i);  //10 means leave blank
          //first space, special character
          if(!i){  //top line
            if(recently_reset)
              lcd.write('R');
            else
              lcd.write(SPACE);
          }else{  //bottom line
            if(eeprom.extras)
              lcd.write(DAY_OF_WEEK);
            else
              lcd.write(SPACE);
          }
          write_number(hour,i);  //write second digit of hour
          lcd.write(LEFT_DOT);  //write left half of colon
          lcd.write(RIGHT_DOT);  //write right half of colon
          write_number(minute_tens,i);  //write first digit of minutes
          //second space, special character
          if(!i){  //top line
            if(eeprom.extras)
              lcd.write(SMALL_AMPM);
            else
              lcd.write(SPACE);
          }else{  //bottom line
            if(eeprom.extras)
              lcd.write(SMALL_SOP);
            else
              lcd.write(SPACE);
          }
          write_number(minute,i);  //write second digit of minutes
        }
        break;

      case TIME_SET_MODE:

        strcpy_P(outstring, lcdtimeset_s);
        print_top(outstring);
        strcpy_P(outstring2, lcdtimesetdisp_s);
        sprintf(outstring,outstring2,day,time.hour,time.minute,time.second);
        print_bot(outstring);

        if(adjust<16)
          lcd.setCursor(adjust,0);
        else
          lcd.setCursor(adjust-16,1);
        lcd.blink();

        break;

      case ONETIME_ALARM_SET_MODE:
        strcpy_P(onoff, (char*)pgm_read_word(&(onoff_s[alarm_onetime.enabled])));

        strcpy_P(outstring, lcdoalmset_s);
        print_top(outstring);
        strcpy_P(outstring2, alarm_set_s);
        sprintf(outstring,outstring2,onoff,alarm_onetime.hour,alarm_onetime.minute,time.hour,time.minute);
        print_bot(outstring);

        if(adjust<=16)
          lcd.setCursor(adjust,0);
        else
          lcd.setCursor(adjust-16,1);
        lcd.blink();

        break;

      case PERSISTENT_ALARM_SET_MODE:

        strcpy_P(outstring, lcdpalmset_s);
        print_top(outstring);
        strcpy_P(onoff, (char*)pgm_read_word(&(onoff_s[alarm_persistent.enabled])));
        strcpy_P(outstring2, alarm_set_s);
        sprintf(outstring,outstring2,onoff,alarm_persistent.hour,alarm_persistent.minute,time.hour,time.minute);
        print_bot(outstring);

        if(adjust<=16)
          lcd.setCursor(adjust,0);
        else
          lcd.setCursor(adjust-16,1);
        lcd.blink();

        break;

      case ALARM_TURNOFF:
      case ALARM_SNOOZE_TURNOFF:

        strcpy_P(outstring2, alarm_active_t_s);
        sprintf(outstring,outstring2,operand1);
        print_top(outstring);

        strcpy_P(outstring2, alarm_active_b_s);
        sprintf(outstring,outstring2,operand2,guess);
        print_bot(outstring);

        lcd.setCursor(adjust-16,1);
        lcd.blink();
        break;
      default:
        error;
    }//switch (time_mode)
  }
}//void display_time()

//display the tetris screen
void display_tetris(){
  char yspot,hspot;
  byte lcd_index;
  unsigned mask;
  byte twosixtemp;

  //if thicker, double the width and height of the board on to the board_thick board
  if(eeprom.thicker){
    zero_outs(board_thick,sizeof(board_thick));

    for(byte y=0; y<MAX_BOARD_HEIGHT/2; y++){
      mask = 0x10;
      for(unsigned x=0; x<8; x++){
        if(board[y] & mask){
          board_thick[y*2] |= bit(x*2) | bit(x*2+1);
          board_thick[y*2+1] |= bit(x*2) | bit(x*2+1);
        }
        mask<<=1;
      }
    }
  }


  //we have 8 custom characters, 4 for the piece and 4 for the fallen pieces, or 2 for the pieces and 6 for the fallen pieces if in two_six mode

  //loop for each row
  for(byte lcd_row=0; lcd_row<2; lcd_row++){
    lcd.setCursor(0,lcd_row);
    lcd_index = 0;

    if(eeprom.thicker){
      yspot = (piece_y*2)/5;
      hspot = ((stack_height-1)*2)/5;
      if(!((stack_height+2)%5))  //the numbers 3,8,13,18,etc. are off and need 1 added
        hspot++;
    }else{
      yspot = piece_y/5;
      hspot = (stack_height-1)/5;
    }

    if(yspot <= hspot)
      yspot = hspot + 1;

    //first print out blank spaces until the piece is reached (might not print out any at all)
    if(eeprom.two_six)
      twosixtemp = yspot;
    else
      twosixtemp = yspot+1;

    for(char i = 15; i > twosixtemp; i--){
      if(!lcd_row && !lcd_index){   //display the number of lines cleared so far
        lcd.print(lines_cleared,DEC);
        if(lines_cleared > 9){
          lcd_index++;
          i--;
        }
      }else{
        lcd.write(' ');
      }
      lcd_index++;
    }

    //print out the two (or one) top line characters for the piece
    if(yspot < 15 && !eeprom.two_six){
      create_custom(lcd_row,yspot+1);
      lcd.createChar(lcd_row*4+0,custom);
      lcd.setCursor(lcd_index,lcd_row);
      lcd.write(lcd_row*4+0);
      lcd_index++;
    }
    if(eeprom.two_six)
      twosixtemp = lcd_row*4+0;
    else
      twosixtemp = lcd_row*4+1;
    create_custom(lcd_row,yspot);
    lcd.createChar(twosixtemp,custom);
    lcd.setCursor(lcd_index,lcd_row);
    lcd.write(twosixtemp);
    lcd_index++;


    //print out blanks spaces until we reach the fallen pieces
    for(char i = yspot-1; i > hspot; i--){
      lcd.write(' ');
      lcd_index++;
    }

    //print out two (or three) characters worth of fallen pieces
    if(eeprom.two_six)
      twosixtemp = 1;
    else
      twosixtemp = 0;
    create_custom(lcd_row,hspot);
    lcd.createChar(lcd_row*4+2-twosixtemp,custom);
    lcd.setCursor(lcd_index,lcd_row);
    lcd.write(lcd_row*4+2-twosixtemp);
    lcd_index++;

    if(hspot > 0){
      create_custom(lcd_row,hspot-1);
      lcd.createChar(lcd_row*4+3-twosixtemp,custom);
      lcd.setCursor(lcd_index,lcd_row);
      lcd.write(lcd_row*4+3-twosixtemp);
      lcd_index++;
    }

    if(hspot > 1 and eeprom.two_six){
      create_custom(lcd_row,hspot-2);
      lcd.createChar(lcd_row*4+3,custom);
      lcd.setCursor(lcd_index,lcd_row);
      lcd.write(lcd_row*4+3);
      lcd_index++;
    }

    //print out black characters until bottom is reached
    for(char i=hspot-2-twosixtemp; i>=0; i--){
      lcd.write(0xFF);
      lcd_index++;
    }
  }  //for(byte lcd_row=0; lcd_row<2; lcd_row++)
}  //void display_screen(){

//make the actual adjustment to time, alarms, and alarm-off guess
void adjust_time(char offset){
  switch(time_mode){
    case TIME_SET_MODE:
      switch(set_time_place){
        case SET_MODE:
          time_mode = (time_mode_enum)(mod((byte)time_mode + offset,1,3));
          adjust_for_display();
          break;
        case SET_DAY_ONOFF:
          time.day = (day_enum)(time.day + offset);
          time.day = (day_enum)mod(time.day,0,6);
          break;
        case SET_HOUR10:
          time.hour += 10 * offset;
          time.hour = mod(time.hour,0,23);
          break;
        case SET_HOUR01:
          time.hour += offset;
          time.hour = mod(time.hour,0,23);
          break;
        case SET_MINUTE10:
          time.minute += 10 * offset;
          time.minute = mod(time.minute,0,59);
          break;
        case SET_MINUTE01:
          time.minute += offset;
          time.minute = mod(time.minute,0,59);
          break;
        case SET_SECOND10:
          time.second += 10 * offset;
          time.second = mod(time.second,0,59);
          break;
        case SET_SECOND01:
          time.second += offset;
          time.second = mod(time.second,0,59);
          break;
        case SET_TENTHS:
          time.ms += 100 * offset;
          time.ms = mod(time.ms,0,999);
          break;
        case SET_HUNDREDTHS:
          time.ms += 10 * offset;
          time.ms = mod(time.ms,0,999);
          break;
        default:
          error;
      }//switch(set_time_place)
      break;
    case ONETIME_ALARM_SET_MODE:
      switch(set_time_place){
        case SET_MODE:
          time_mode = (time_mode_enum)(mod((byte)time_mode + offset,1,3));
          adjust_for_display();
          break;
        case SET_DAY_ONOFF:
          alarm_onetime.enabled += offset;
          alarm_onetime.enabled = mod(alarm_onetime.enabled,0,1);
          break;
        case SET_HOUR10:
          alarm_onetime.hour += 10 * offset;
          alarm_onetime.hour = mod(alarm_onetime.hour,0,23);
          alarm_onetime.enabled = true;
          break;
        case SET_HOUR01:
          alarm_onetime.hour += offset;
          alarm_onetime.hour = mod(alarm_onetime.hour,0,23);
          alarm_onetime.enabled = true;
          break;
        case SET_MINUTE10:
          alarm_onetime.minute += 10 * offset;
          alarm_onetime.minute = mod(alarm_onetime.minute,0,59);
          alarm_onetime.enabled = true;
          break;
        case SET_MINUTE01:
          alarm_onetime.minute += offset;
          alarm_onetime.minute = mod(alarm_onetime.minute,0,59);
          alarm_onetime.enabled = true;
          break;
        default:
          error;
      }//switch(set_time_place)
      break;
    case PERSISTENT_ALARM_SET_MODE:
      switch(set_time_place){
        case SET_MODE:
          time_mode = (time_mode_enum)(mod((byte)time_mode + offset,1,3));
          adjust_for_display();
          break;
        case SET_DAY_ONOFF:
          alarm_persistent.enabled += offset;
          alarm_persistent.enabled = mod(alarm_persistent.enabled,0,1);
          break;
        case SET_HOUR10:
          alarm_persistent.hour += 10 * offset;
          alarm_persistent.hour = mod(alarm_persistent.hour,0,23);
          alarm_persistent.enabled = true;
          break;
        case SET_HOUR01:
          alarm_persistent.hour += offset;
          alarm_persistent.hour = mod(alarm_persistent.hour,0,23);
          alarm_persistent.enabled = true;
          break;
        case SET_MINUTE10:
          alarm_persistent.minute += 10 * offset;
          alarm_persistent.minute = mod(alarm_persistent.minute,0,59);
          alarm_persistent.enabled = true;
          break;
        case SET_MINUTE01:
          alarm_persistent.minute += offset;
          alarm_persistent.minute = mod(alarm_persistent.minute,0,59);
          alarm_persistent.enabled = true;
          break;
        default:
          error;
      }//switch(set_time_place)
      break;
    case ALARM_TURNOFF:
    case ALARM_SNOOZE_TURNOFF:
      switch(set_time_place){
        case SET_MODE:
          guess += 1000 * offset;
          guess = mod(guess,0,9999);
          break;
        case SET_DAY_ONOFF:
          guess += 100 * offset;
          guess = mod(guess,0,9999);
          break;
        case SET_HOUR10:
          guess += 10 * offset;
          guess = mod(guess,0,9999);
          break;
        case SET_HOUR01:
          guess += 1 * offset;
          guess = mod(guess,0,9999);
          break;
        case SET_MINUTE10:
          guess = 0;
          break;
        case SET_MINUTE01:
          guess = result;
          break;
        default:
          error;
      }//switch(set_time_place)
      break;
    default:
      error;
  }//switch(time_mode)
}  //void adjust_time(char offset)

//returns how many spaces need to be displayed so the arrow indicating which field is being set is correct
void adjust_for_display(){
  switch(time_mode){
    case ALARM_TURNOFF:
    case ALARM_SNOOZE_TURNOFF:
      stops[SET_MODE]=21;
      stops[SET_DAY_ONOFF]=22;
      stops[SET_HOUR10]=23;
      stops[SET_HOUR01]=24;
      break;
    case ONETIME_ALARM_SET_MODE:
    case PERSISTENT_ALARM_SET_MODE:
      stops[SET_MODE]=5;
      stops[SET_DAY_ONOFF]=18;
      stops[SET_HOUR10]=20;
      stops[SET_HOUR01]=21;
      stops[SET_MINUTE10]=23;
      stops[SET_MINUTE01]=24;
      break;
    case TIME_SET_MODE:
      stops[SET_MODE]=5;
      stops[SET_DAY_ONOFF]=17;
      stops[SET_HOUR10]=20;
      stops[SET_HOUR01]=21;
      stops[SET_MINUTE10]=23;
      stops[SET_MINUTE01]=24;
      stops[SET_SECOND10]=26;
      stops[SET_SECOND01]=27;
      stops[SET_TENTHS]=28;
      stops[SET_HUNDREDTHS]=29;
      break;
    default:
      //error;
      print_error(time_mode,69);
  }//switch(time_mode)

}  //void adjust_for_display()

/*
byte smiley[8] = {
  B00000,
  B10001,
  B00000,
  B00000,
  B10001,
  B01110,
  B00000,
  B00000,
};
*/
//2 rows, top 0, bottom 1; 16 columns, 15 left most
void create_custom(byte row, byte column){
  if(eeprom.thicker)
    create_custom_thick(row, column);
  else
    create_custom_thin(row, column);
}  //void create_custom(byte row, byte column){

//2 rows, top 0, bottom 1; 16 columns, 15 left most
void create_custom_thin(byte row, byte column){
  byte custom_mask;
  unsigned mask;
  unsigned convert_mask_from,convert_mask_to;
  byte from;
  byte y_temp;

  zero_outs(custom,sizeof(custom));
  custom_mask = B10000;

  y_temp = column*5;

  //start with upper board row, left bits of custom char
  for(char y=y_temp+4; y>=y_temp; y--){  //adjust to the place on the board where the wanted column is
    mask = B1 << (row*8);  //adjust for the particular row
    for(char i=0;i<8;i++){
        if(board[y] & mask)  //if the bit is set on the board
          custom[i] |= custom_mask;  //then set it for the custom character
      mask <<= 1;
    }
    custom_mask >>= 1;
  }
}  //void create_cutom_thin(byte row, byte column)

//2 rows, top 0, bottom 1; 16 columns, 15 left most
void create_custom_thick(byte row, byte column){
  byte custom_mask;
  unsigned mask;
  unsigned convert_mask_from,convert_mask_to;
  byte from;
  byte y_temp;

  zero_outs(custom,sizeof(custom));
  custom_mask = B10000;

  y_temp = column*5;

  //start with upper board row, left bits of custom char
  for(char y=y_temp+4; y>=y_temp; y--){  //adjust to the place on the board where the wanted column is
    mask = B1 << (row*8);  //adjust for the particular row
    for(char i=0;i<8;i++){
        if(board_thick[y] & mask)  //if the bit is set on the board
          custom[i] |= custom_mask;  //then set it for the custom character
      mask <<= 1;
    }
    custom_mask >>= 1;
  }
}  //void create_cutom_thick(byte row, byte column)

SIGNAL(TIMER1_OVF_vect){
   byte button_number;

   if (!(button_timer++ & BUTTON_SCALER)){

     if(button_pressed_count <= 10){//don't let the total number of button presses go over 10 in case a button is acidentaly held down for an extended period of time

       button_count[BUTTON_UP] <<= 1;  //make room for the next read of the wire and pad with 0
       if(digitalRead(BUTTON_UP_PIN) == LOW) //low means button pressed, otherwise just leave as 0, not pressed
          button_count[BUTTON_UP] |= 0x01;

       button_count[BUTTON_DOWN] <<= 1;  //make room for the next read of the wire and pad with 0
       if(digitalRead(BUTTON_DOWN_PIN) == LOW) //low means button pressed, otherwise just leave as 0, not pressed
          button_count[BUTTON_DOWN] |= 0x01;

       button_count[BUTTON_LEFT] <<= 1;  //make room for the next read of the wire and pad with 0
       if(digitalRead(BUTTON_LEFT_PIN) == LOW) //low means button pressed, otherwise just leave as 0, not pressed
          button_count[BUTTON_LEFT] |= 0x01;

       button_count[BUTTON_RIGHT] <<= 1;  //make room for the next read of the wire and pad with 0
       if(digitalRead(BUTTON_RIGHT_PIN) == LOW) //low means button pressed, otherwise just leave as 0, not pressed
          button_count[BUTTON_RIGHT] |= 0x01;

       for(button_number=0; button_number<4; button_number++){
          //if a button has been pressed the required number of times and is not already pressed, then declare it pressed
          if((button_count[button_number] & BUTTON_MASK) == BUTTON_MASK && !(last_button & _BV(button_number))){
             current_button |= _BV(button_number);
             buttons[button_number]++;
             button_pressed_count++;
          //if a button has not been pressed the required number of time and is currently pressed, then delcare it not pressed
          }else if((button_count[button_number] & BUTTON_MASK) == 0 && (last_button & _BV(button_number))){
             current_button &= ~_BV(button_number);
          }
       }
      last_button=current_button;

     }//if(button_pressed_count <= 10)

   }//if (!(button_timer++ & BUTTON_SCALER))
}
#define FILE_LENGTH 1368
